package org.nakedosgi.processor;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.jar.Manifest;
import javax.annotation.processing.Filer;
import javax.tools.FileObject;
import javax.tools.JavaFileManager.Location;
import javax.tools.StandardLocation;

/**
 * Utilities for working with the manifest in annotation processing scenarios
 * @author Evan Cowden
 */
public class ManifestUtil {

    public static final String MANIFEST_PATH = "META-INF/MANIFEST.MF";
    public static final String MANIFEST_BEGINNING = "Manifest-Version: 1.0\n";

    /**
     * Get a manifest.  If an old one exists, load it.  Otherwise, create
     * a new one.
     * @param filer
     * @return a manifest reflecting the current state of the project
     */
    public static Manifest readManifest(Filer filer) throws IOException {
        FileObject file = getResource(filer, StandardLocation.CLASS_OUTPUT, "", MANIFEST_PATH);

        Manifest result = new Manifest();

        // If the manifest already exists, read its contents
        if (file != null) {
            InputStream in = file.openInputStream();
            try {
                result.read(in);
            } finally {
                in.close();
            }
        }

        return result;
    }

    /**
     * Write the specified manifest using the given filer
     * @param filer
     * @param manifest
     */
    public static void writeManifest(Filer filer, Manifest manifest) throws IOException {
        FileObject file = filer.createResource(StandardLocation.CLASS_OUTPUT, "", MANIFEST_PATH);

        OutputStream out = file.openOutputStream();
        try {
            manifest.write(out);
        } finally {
            out.close();
        }
    }

    /**
     * Get an existing resource, or null if it does not exist
     * @param filer
     * @param location
     * @param pkg
     * @param relativeName
     * @throws java.io.IOException
     * @return the requested resource, or null if it does not exist
     */
    protected static FileObject getResource(Filer filer, Location location, CharSequence pkg, CharSequence relativeName) throws IOException {
        FileObject file = filer.getResource(location, pkg, relativeName);
        boolean exists;
        try {
            if (file.getCharContent(true).length() > 0) {
                exists = true;
            } else {
                exists = false;
            }
        } catch (FileNotFoundException ex) {
            exists = false;
        }

        if (exists) {
            return file;
        }

        return null;
    }
}
