/*
 * Copyright 2008 Evan Cowden
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */
package org.nakedosgi.processor;

import java.io.IOException;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;
import java.util.concurrent.ConcurrentHashMap;
import java.util.jar.Attributes;
import java.util.jar.Manifest;
import javax.annotation.processing.ProcessingEnvironment;
import org.nakedosgi.manifest.BasicManifestBuilder;

/**
 * Manages the operation of bundle processors and creates the final manifest
 * file when all are complete.
 * @author Evan Cowden
 */
public class OSGiProcessorManager {

    /**
     * The singleton instance
     */
    private static OSGiProcessorManager instance = new OSGiProcessorManager();
    private final SortedMap<String, String> manifest = new TreeMap();
    private final Map<OSGiProcessor, Boolean> processors = new ConcurrentHashMap();
    private final Map<String, Object> metainfo = new ConcurrentHashMap();
    private int finishedProcessors = 0;
    ProcessingEnvironment processingEnv;

    /**
     * Prevent the instantiation of an instance outside of the singleton
     */
    private OSGiProcessorManager() {
        //Add basic manifest info, such as version, to manifest
        BasicManifestBuilder.build(manifest);
    }

    /**
     * Get an instance of the {@code OSGiProcessorManager}
     * @return
     */
    public static OSGiProcessorManager getInstance() {
        return instance;
    }

    /**
     * Register a {@code OSGiProcessor}'s existence
     * @param processor
     */
    protected void registerBundleProcessor(OSGiProcessor processor, ProcessingEnvironment processingEnv) {
        processors.put(processor, Boolean.FALSE);
        this.processingEnv = processingEnv;  //Not great, but they're all the same...
    }

    /**
     * Notify the manager that a processor is finished processing
     * @param processor
     */
    protected synchronized void processorFinished(OSGiProcessor processor) {
        boolean finished = processors.get(processor);
        if (finished == false) {
            processors.put(processor, true);
            finishedProcessors++;
        }

        if (finishedProcessors == processors.size()) {
            finished();
        }
    }

    /**
     * Called when all processors are finished processing, write the manifest
     * and perform other finalization
     */
    protected void finished() {
        //Give all processors a chance to finalize
        for (OSGiProcessor processor : processors.keySet()) {
            processor.processingDone();
        }

        //Write the manifest
        try {
            writeManifest();
        } catch (IOException ex) {
            System.out.println("Error writing manifest file");
            ex.printStackTrace();
        }
    }

    /**
     * Get the manifest
     * @return
     */
    public Map<String, String> getManifest() {
        return manifest;
    }

    /**
     * Get the meta-information map.  This can be used to share state
     * amongst processors.
     */
    public Map<String, Object> getMetainfo() {
        return metainfo;
    }

    /**
     * Write the manifest file, using the instance's manifest map as main
     * attribute entries
     * @throws java.io.IOException
     */
    protected void writeManifest() throws IOException {

        //Assemble standard JAR manifest
        Manifest manifestObject = ManifestUtil.readManifest(processingEnv.getFiler());
        Attributes mainAttributes = manifestObject.getMainAttributes();
        for (Map.Entry<String, String> entry : manifest.entrySet()) {
            mainAttributes.putValue(entry.getKey(), entry.getValue());
        }

        //Write manifest
        ManifestUtil.writeManifest(processingEnv.getFiler(), manifestObject);
    }
}
