/*
 * Copyright 2008 Evan Cowden
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */
package org.nakedosgi.processor.scr;

import java.io.IOException;
import java.io.OutputStream;
import java.util.Collection;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import javax.annotation.processing.Filer;
import javax.annotation.processing.ProcessingEnvironment;
import javax.lang.model.element.Element;
import javax.lang.model.element.TypeElement;
import javax.tools.FileObject;
import javax.tools.StandardLocation;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import org.nakedosgi.Constants;
import org.nakedosgi.manifest.ManifestConstants;
import org.nakedosgi.processor.OSGiProcessorManager;
import org.nakedosgi.processor.ParsingUtil;
import org.nakedosgi.processor.scr.model.Component;

/**
 * Utilities for parsing 
 * @author Evan Cowden
 */
public class SCRProcessingUtil {

    private static final String COMPONENTS_KEY = "SCRComponents";

    /**
     * Retrieve the shared components collection.
     */
    public static Map<String, Component> getComponents() {
        //Retrieve the collection from the OSGiProcessorManager metainfo collection
        OSGiProcessorManager manager = OSGiProcessorManager.getInstance();
        Map<String, Object> metainfo = manager.getMetainfo();
        Map<String, Component> result = (Map<String, Component>) metainfo.get(COMPONENTS_KEY);

        //If it does not exist, create it
        if (result == null) {
            result = new ConcurrentHashMap();
            metainfo.put(COMPONENTS_KEY, result);
        }

        return result;
    }

    /**
     * Get the specified component out of the components collection.
     * If it does not already exist, create it and add it to the collection.
     * @param componentClass
     * @param components
     * @return
     */
    public static Component getComponent(String componentClass, Map<String, Component> components) {
        Component result = components.get(componentClass);

        if (result == null) {
            result = new Component();
            components.put(componentClass, result);
        }

        return result;
    }

    /**
     * Get the component out of the components collection that would contain the 
     * the specified element.  If it does not already exist, create it and add 
     * it to the collection.
     * @param childElement
     * @param components
     * @return
     */
    public static Component getComponent(Element childElement, Map<String, Component> components) {
        TypeElement typeElement = ParsingUtil.getEnclosingType(childElement);

        String componentClass = typeElement.getQualifiedName().toString();

        return getComponent(componentClass, components);
    }

    /**
     * Write component descriptors for the specified components
     * @param components
     */
    public static void writeComponentDescriptors(Collection<Component> components, ProcessingEnvironment processingEnv) throws JAXBException, IOException {
        JAXBContext context = JAXBContext.newInstance(Component.class);
        Marshaller marshaller = context.createMarshaller();
        marshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE); //Set pretty print to true
        Filer filer = processingEnv.getFiler();


        for (Component component : components) {
            String filename = getFilename(component.getName());
            FileObject file = filer.createResource(
                    StandardLocation.CLASS_OUTPUT,
                    "",
                    filename); //TODO source doodad

            OutputStream out = file.openOutputStream();
            try {
                marshaller.marshal(component, out);
            } finally {
                out.close();
            }
        }
    }

    /**
     * Create a filename for an SCR component
     * @param componentName
     * @return
     */
    public static String getFilename(String componentName) {
        StringBuilder builder = new StringBuilder();
        builder.append(Constants.SERVICE_DEFINITION_DIRECTORY);
        builder.append('/');
        builder.append(componentName);
        builder.append(".xml");

        return builder.toString();
    }

    /**
     * Extract information about generated SCR component definitions and 
     * place appropriate entries into the manifest.
     * @param manifest
     * @param components
     */
    public static void addSCRComponents(Map<String, String> manifest, Collection<Component> components) {
        //Build the value string
        StringBuilder builder = new StringBuilder();
        for (Component component : components) {
            String filename = SCRProcessingUtil.getFilename(component.getName());
            builder.append(filename);
            builder.append(ManifestConstants.MANIFEST_SEPERATOR);
        }

        //...remove the last ", "
        int index = builder.lastIndexOf(ManifestConstants.MANIFEST_SEPERATOR);
        if (index > 0) {
            builder.delete(index, index + ManifestConstants.MANIFEST_SEPERATOR.length());
        }

        //Add to manifest
        manifest.put(ManifestConstants.SERVICE_COMPONENT_KEY, builder.toString());
    }
}
